package fr.inria.ble_provider

import android.Manifest
import android.content.Context
import android.app.Activity
import android.content.pm.PackageManager
import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat.requestPermissions
import androidx.core.content.ContextCompat

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.indoorlocation.core.IndoorLocationProviderListener
import io.indoorlocation.polestarlocationprovider.PolestarIndoorLocationProvider

/** BleProviderPlugin */
public class BleProviderPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  private lateinit var context: Context
  private lateinit var activity: Activity
  private lateinit var provider: PolestarIndoorLocationProvider

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "ble_provider")
    channel.setMethodCallHandler(this);
    context = flutterPluginBinding.applicationContext
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "ble_provider")
      channel.setMethodCallHandler(BleProviderPlugin())
    }
  }

  override fun onDetachedFromActivity(): Unit {
    print("detached")
  }

  override fun onReattachedToActivityForConfigChanges(@NonNull p0: ActivityPluginBinding): Unit {
    print("reattached to activity")
  }

  override fun onAttachedToActivity(@NonNull binding: ActivityPluginBinding): Unit {
    print("attached to activity")
    this.activity = binding.activity;
  }

  override fun onDetachedFromActivityForConfigChanges(): Unit {
    print("detached from activity")
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
      "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
      "init" -> {
        this.provider = PolestarIndoorLocationProvider(context, call.argument("key"))
        when {
          ContextCompat.checkSelfPermission(
                  this.context,
                  Manifest.permission.ACCESS_COARSE_LOCATION
          ) == PackageManager.PERMISSION_GRANTED -> {
            // You can use the API that requires the permission.
          }
          else -> {
            requestPermissions(this.activity,
                    arrayOf(Manifest.permission.ACCESS_COARSE_LOCATION),
                    42)
          }
        }
        result.success(true)
      }
      "setLocationCallback" -> {
        this.provider.addListener( FlutterLocationListener(channel) )
        result.success(this.provider.listeners.size == 1)
      }
      "start" -> {
        this.provider.start()
        this.provider.onSynchronizationSuccess()
        result.success(this.provider.isStarted)
      }
      "stop" -> {
        this.provider.stop()
        result.success(!this.provider.isStarted)
      }
      else -> {
        result.notImplemented()
      }
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }
}
